# Copyright 2013-2023 Lawrence Livermore National Security, LLC and other
# Spack Project Developers. See the top-level COPYRIGHT file for details.
#
# SPDX-License-Identifier: (Apache-2.0 OR MIT)


from spack.package import *
from spack.pkg.builtin.singularityce import SingularityBase


# Apptainer is the new name of Singularity, piggy-back on the original package
class Apptainer(SingularityBase):
    """Apptainer is an open source container platform designed to be simple, fast, and
    secure. Many container platforms are available, but Apptainer is designed for
    ease-of-use on shared systems and in high performance computing (HPC)
    environments.

    Needs post-install chmod/chown steps to enable full functionality.
    See package definition or `spack-build-out.txt` build log for details,
    e.g.:

        tail -15 $(spack location -i apptainer)/.spack/spack-build-out.txt
    """

    homepage = "https://apptainer.org"
    url = "https://github.com/apptainer/apptainer/releases/download/v1.0.2/apptainer-1.0.2.tar.gz"
    git = "https://github.com/apptainer/apptainer.git"

    version("main", branch="main")
    version("1.3.2", sha256="483910727e1a15843b93d9f2db1fc87e27804de9c74da13cc32cd4bd0d35e079")
    # CVE 2024-3727
    version(
        "1.3.1",
        sha256="6956c689c4a8f148789c5c34b33c15ad8f3460b4cee3f48022119fd872eacee9",
        deprecated=True
    )
    version(
        "1.2.2",
        sha256="fbb01542847020a869d6327f1dbc2833f74577db3097936e90e5385f7607d4b9",
        deprecated=True
    )
    version(
        "1.1.9",
        sha256="c615777539154288542cf393d3fd44c04ccb3260bc6330dc324d4e4ebe902bfa",
        deprecated=True
    )
    version(
        "1.1.7",
        sha256="e6d3956a26c3965703402e17f153ba07f59bf710068806462b314d2d04e825e7",
        deprecated=True
    )
    version(
        "1.1.6",
        sha256="5f32d305279a51ce8bdbe69e733c4ac12b1efdcb77758fab8ec9463e96a8fd82",
        deprecated=True
    )
    version(
        "1.1.5",
        sha256="3eadb26b6656a89a111abe29c7e50eab0023e9a8718f1e77e46ca871398bfa67",
        deprecated=True
    )
    version(
        "1.1.4",
        sha256="b1ab9d5842002803e66da8f456ee00f352ea2bb43436d5b668f19ef7475ed4a5",
        deprecated=True
    )
    version(
        "1.1.3",
        sha256="c7bf7f4d5955e1868739627928238d02f94ca9fd0caf110b0243d65548427899",
        deprecated=True
    )
    version(
        "1.0.2",
        sha256="2d7a9d0a76d5574459d249c3415e21423980d9154ce85e8c34b0600782a7dfd3",
        deprecated=True
    )

    depends_on("go@1.17.5:", when="@1.1.0:")
    depends_on("squashfuse", type="run")

    singularity_org = "apptainer"
    singularity_name = "apptainer"
    singularity_security_urls = (
        "https://apptainer.org/docs/admin/main/security.html",
        "https://apptainer.org/docs/admin/main/admin_quickstart.html#apptainer-security",
    )

    # Override config options from SingularityBase
    @property
    def config_options(self):
        spec = self.spec
        options = []
        if spec.satisfies("@1.1.0: +suid"):
            options.append("--with-suid")
        return options

    def flag_handler(self, name, flags):
        # Certain go modules this build pulls in cannot be built with anything
        # other than -O0. Best to just discard any injected flags.
        return (None, flags, None)
