# Copyright 2013-2019 Lawrence Livermore National Security, LLC and other
# Spack Project Developers. See the top-level COPYRIGHT file for details.
#
# SPDX-License-Identifier: (Apache-2.0 OR MIT)

from spack import *

import os

class Buildah(MakefilePackage):
    """The Buildah package provides a command line tool that can be used to

    * create a working container, either from scratch or using an image as a starting point
    * create an image, either from a working container or via the instructions in a Dockerfile
    * images can be built in either the OCI image format or the traditional upstream docker image format
    * mount a working container's root filesystem for manipulation
    * unmount a working container's root filesystem
    * use the updated contents of a container's root filesystem as a filesystem layer to create a new image
    * delete a working container or an image
    * rename a local container
"""

    homepage = "https://buildah.io/"
    url      = "https://github.com/containers/buildah/archive/v1.14.5.tar.gz"

    version("1.35.4", sha256="7d655453d6ff150503a4ecd61481622d83735f89fc69cf1805f0d3a1619c8c2c")
    # CVE-2024-3727
    version(
        "1.31.2",
        sha256="03137dc0952c242cfe8030d1667e5459b8f3fd2291a15117592fcf974ba09f8b",
        deprecated=True
    )
    version(
        '1.29.1',
        sha256='ac20cdbd81616f3f03ef42dc3139de118e3b6d0ed737c994c45dc1039e0fa1c1',
        deprecated=True
    )
    version(
        '1.15.0',
        sha256='ffa3b84f3ef1cc482a3e9e27e8f7417e4dd5c70bee7c6208155b9e2039b1d606',
        deprecated=True
    )
    version(
        '1.14.9',
        sha256='fa084af683a9f91bdc0fe24077ad56ec1bc9f4f98445ddcba4e0796710281adf',
        deprecated=True
    )
    version(
        '1.14.5',
        sha256='74633280c814d340ed32653106bc706f306cf78afb57bb51c3cdca8893d95bd4',
        deprecated=True
    )
    version(
        '1.14.0',
        sha256='3eb637c5a5b35ba865886d20ce937e7c6b2929573c03af09a86c3be11151ec80',
        deprecated=True
    )
   
    variant(
        "network",
        default="netavark",
        multi=False,
        values=(
            "cni",
            conditional("netavark", when="@1.24:")
        ),
    )

    depends_on('bzip2')
    depends_on('go',type='build')
    depends_on('go-md2man', type='build')
    depends_on('gpgme')
    depends_on('libassuan')
    depends_on('libgpg-error')
    depends_on('lvm2')
    depends_on('libseccomp')
    depends_on('runc@1.0:', type='run')

    depends_on('cni-plugins', type='run', when='network=cni')
    depends_on('netavark', type='run', when='network=netavark')

    # ALCF config
    depends_on("fuse-overlayfs", type="run")

    def patch(self):
        defs = FileFilter("vendor/github.com/containers/common/pkg/config/default.go")
        config_go = FileFilter(
            "vendor/github.com/containers/common/pkg/config/config_{}.go".format(
                self.spec.platform
            )
        )

        # Prepend the provided runc executable to podman's built-in runc search path
        defs.filter('"runc": {', '"runc": {' + '"{0}",'.format(self.spec["runc"].prefix.sbin.runc))
        if self.spec.satisfies("network=cni"):
                # Prepend the provided cni-plugins directory to the cni-plugin search path
                defs.filter(
                    r"DefaultCNIPluginDirs = \[\]string{",
                    "DefaultCNIPluginDirs = []string{"
                    + '\n        "{0}",'.format(self.spec["cni-plugins"].prefix.bin),
                )
                # Set the default path for slirp4netns to the provided slirp4netns executable
                defs.filter(
                    "cniConfig := _cniConfigDir",
                    "cniConfig := _cniConfigDir"
                    + '\n        defaultEngineConfig.NetworkCmdPath = "{0}"'.format(
                	self.spec["slirp4netns"].prefix.bin.slirp4netns
                    ),
                )
        if self.spec.satisfies("network=netavark"):
                # Prepend the provided netavark directory to the netavark plugin search path
                defs.filter(
                    r"DefaultNetavarkPluginDirs = \[\]string{",
                    "DefaultNetavarkPluginDirs = []string{"
                    + '\n        "{0}",'.format(self.spec["netavark"].prefix.bin),
                )
                # Prepend netavark libexec/podman directory to default helper binary search path
                config_go.filter(
                    r"defaultHelperBinariesDir = \[\]string{",
                    "defaultHelperBinariesDir = []string{"
                    + '\n    "{0}",'.format(self.spec["netavark"].prefix.libexec.podman)
                )

        # modify `const` paths to /etc here since `const` can't be changed at build time
        # containers.conf
        config_go.filter(
            r"OverrideContainersConfig =.*",
            'OverrideContainersConfig = "{}/etc/containers/containers.conf"'.format(
                self.spec.prefix,
            ),
        )
        # seccomp.json
        defs.filter(
            r"SeccompOverridePath =.*",
            'SeccompOverridePath = "{}/etc/containers/seccomp.json"'.format(
                self.spec.prefix,
            ),
        )


    def setup_build_environment(self, env):
        env.set(
            'EXTRA_LDFLAGS',
            (
                # mounts.conf
                " -X github.com/containers/common.subscriptions.OverrideMountsFile={0}/etc/containers/mounts.conf"
                # policy.json
                " -X github.com/containers/image/v5/signature.systemDefaultPolicyPath={0}/etc/containers/policy.json"
                # registries.conf(.d)
                " -X github.com/containers/image/v5/sysregistriesv2.systemRegistriesConfPath={0}/etc/containers/registries.conf"
                " -X github.com/containers/image/v5/sysregistriesv2.systemRegistriesConfDirPath={0}/etc/containers/registries.conf.d"
                # storage.conf
                " -X github.com/containers/storage/types.defaultOverrideConfigFile={0}/etc/containers/storage.conf"
            ).format(self.prefix)
        )

        env.set('ETCDIR', '{}/etc'.format(self.prefix))
        tags = "seccomp exclude_graphdriver_devicemapper exclude_graphdriver_btrfs"
        if self.spec.satisfies("network=cni"):
            tags += " cni"
        env.set('BUILDTAGS', tags)


    def install(self, spec, prefix):
        make('install', 'PREFIX={0}'.format(prefix))
